import time

from sanic import Sanic


class SanicStatsD:
    def __init__(self, app=None, statsd=None, tags=None, metric='api.response.time'):
        self.statsd = statsd
        self.tags = tags
        self.metric = metric
        # If an app was provided, then call `init_app` for them
        if app is not None:
            self.init_app(app)
        else:
            self.app = None

    def init_app(self, app: Sanic):
        """hook on request start etc."""
        app.register_middleware(self.statsd_start_timers, 'request')
        app.register_middleware(self.statsd_submit_timers, 'response')

    async def statsd_start_timers(self, request):
        request['started_at'] = time.time()

    async def statsd_submit_timers(self, request, response):
        elapsed = time.time() - request['started_at']
        tags = [
            'method:{}'.format(request.method.lower()),
            'uri_template:{}'.format(request.uri_template)
        ]
        if self.tags:
            tags.extend(self.tags)
        # TODO - implement support for https://github.com/jsocol/pystatsd
        # (impediment: what to to with tags?)
        self.statsd.histogram(self.metric, elapsed, tags=tags)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function

import io
from glob import glob
from os.path import basename
from os.path import dirname
from os.path import join
from os.path import splitext

from setuptools import setup


def read(*names, **kwargs):
    return io.open(
        join(dirname(__file__), *names),
        encoding=kwargs.get('encoding', 'utf8')
    ).read()


setup(
    name='sanic-statsd',
    version='0.1.2',
    license='MIT license',
    description='A statsd plugin for Sanic',
    # long_description='%s\n%s' % (
    #     re.compile('^.. start-badges.*^.. end-badges', re.M | re.S).sub('', read('README.rst')),
    #     re.sub(':[a-z]+:`~?(.*?)`', r'``\1``', read('CHANGELOG.rst'))
    # ),
    author='Sergei Beilin',
    author_email='saabeilin@gmail.com',
    url='https://github.com/saabeilin/sanic-statsd',
    # packages=find_packages('src'),
    packages=['sanic_statsd'],
    package_dir={'': 'src'},
    py_modules=[splitext(basename(path))[0] for path in glob('src/*.py')],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        # complete classifier list: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Utilities',
    ],
    keywords=[
        'sanic', 'statsd', 'datadog'
    ],
    install_requires=[
        'sanic'
    ]
)
